@@ -4,17 +4,11 @@ class ContextMenusController < ApplicationController
 
   def issues
     @issues = Issue.visible.all(:conditions => {:id => params[:ids]}, :include => :project)
-
     if (@issues.size == 1)
       @issue = @issues.first
-      @allowed_statuses = @issue.new_statuses_allowed_to(User.current)
-    else
-      @allowed_statuses = @issues.map do |i|
-        i.new_statuses_allowed_to(User.current)
-      end.inject do |memo,s|
-        memo & s
-      end
     end
+
+    @allowed_statuses = @issues.map(&:new_statuses_allowed_to).reduce(:&)
     @projects = @issues.collect(&:project).compact.uniq
     @project = @projects.first if @projects.size == 1
 
@@ -34,14 +28,26 @@ class ContextMenusController < ApplicationController
       @trackers = @project.trackers
     else
       #when multiple projects, we only keep the intersection of each set
-      @assignables = @projects.map(&:assignable_users).inject{|memo,a| memo & a}
-      @trackers = @projects.map(&:trackers).inject{|memo,t| memo & t}
+      @assignables = @projects.map(&:assignable_users).reduce(:&)
+      @trackers = @projects.map(&:trackers).reduce(:&)
     end
 
     @priorities = IssuePriority.active.reverse
-    @statuses = IssueStatus.find(:all, :order => 'position')
     @back = back_url
 
+    @options_by_custom_field = {}
+    if @can[:edit]
+      custom_fields = @issues.map(&:available_custom_fields).reduce(:&).select do |f|
+        %w(bool list user version).include?(f.field_format) && !f.multiple?
+      end
+      custom_fields.each do |field|
+        values = field.possible_values_options(@projects)
+        if values.any?
+          @options_by_custom_field[field] = values
+        end
+      end
+    end
+
     render :layout => false
   end
 
