@@ -19,6 +19,8 @@ class CustomFieldsController < ApplicationController
   layout 'admin'
 
   before_filter :require_admin
+  before_filter :build_new_custom_field, :only => [:new, :create]
+  before_filter :find_custom_field, :only => [:edit, :update, :destroy]
 
   def index
     @custom_fields_by_type = CustomField.find(:all).group_by {|f| f.class.name }
@@ -26,39 +28,51 @@ class CustomFieldsController < ApplicationController
   end
 
   def new
-    @custom_field = begin
-      if params[:type].to_s.match(/.+CustomField$/)
-        params[:type].to_s.constantize.new(params[:custom_field])
-      end
-    rescue
-    end
-    (redirect_to(:action => 'index'); return) unless @custom_field.is_a?(CustomField)
+  end
 
+  def create
     if request.post? and @custom_field.save
       flash[:notice] = l(:notice_successful_create)
       call_hook(:controller_custom_fields_new_after_save, :params => params, :custom_field => @custom_field)
       redirect_to :action => 'index', :tab => @custom_field.class.name
     else
-      @trackers = Tracker.find(:all, :order => 'position')
+      render :action => 'new'
     end
   end
 
   def edit
-    @custom_field = CustomField.find(params[:id])
-    if request.post? and @custom_field.update_attributes(params[:custom_field])
+  end
+
+  def update
+    if request.put? and @custom_field.update_attributes(params[:custom_field])
       flash[:notice] = l(:notice_successful_update)
       call_hook(:controller_custom_fields_edit_after_save, :params => params, :custom_field => @custom_field)
       redirect_to :action => 'index', :tab => @custom_field.class.name
     else
-      @trackers = Tracker.find(:all, :order => 'position')
+      render :action => 'edit'
     end
   end
 
   def destroy
-    @custom_field = CustomField.find(params[:id]).destroy
+    @custom_field.destroy
     redirect_to :action => 'index', :tab => @custom_field.class.name
   rescue
     flash[:error] = l(:error_can_not_delete_custom_field)
     redirect_to :action => 'index'
   end
+
+  private
+
+  def build_new_custom_field
+    @custom_field = CustomField.new_subclass_instance(params[:type], params[:custom_field])
+    if @custom_field.nil?
+      render_404
+    end
+  end
+
+  def find_custom_field
+    @custom_field = CustomField.find(params[:id])
+  rescue ActiveRecord::RecordNotFound
+    render_404
+  end
 end
