@@ -18,9 +18,9 @@
 class DocumentsController < ApplicationController
   default_search_scope :documents
   model_object Document
-  before_filter :find_project, :only => [:index, :new]
-  before_filter :find_model_object, :except => [:index, :new]
-  before_filter :find_project_from_association, :except => [:index, :new]
+  before_filter :find_project_by_project_id, :only => [:index, :new, :create]
+  before_filter :find_model_object, :except => [:index, :new, :create]
+  before_filter :find_project_from_association, :except => [:index, :new, :create]
   before_filter :authorize
 
   helper :attachments
@@ -49,25 +49,36 @@ class DocumentsController < ApplicationController
   def new
     @document = @project.documents.build
     @document.safe_attributes = params[:document]
-    if request.post? and @document.save	
-      attachments = Attachment.attach_files(@document, params[:attachments])
+  end
+
+  def create
+    @document = @project.documents.build
+    @document.safe_attributes = params[:document]
+    @document.save_attachments(params[:attachments])
+    if @document.save
       render_attachment_warning_if_needed(@document)
       flash[:notice] = l(:notice_successful_create)
       redirect_to :action => 'index', :project_id => @project
+    else
+      render :action => 'new'
     end
   end
 
   def edit
-    @categories = DocumentCategory.active #TODO: use it in the views
+  end
+
+  def update
     @document.safe_attributes = params[:document]
-    if request.post? and @document.save
+    if request.put? and @document.save
       flash[:notice] = l(:notice_successful_update)
       redirect_to :action => 'show', :id => @document
+    else
+      render :action => 'edit'
     end
   end
 
   def destroy
-    @document.destroy
+    @document.destroy if request.delete?
     redirect_to :controller => 'documents', :action => 'index', :project_id => @project
   end
 
@@ -78,11 +89,4 @@ class DocumentsController < ApplicationController
     Mailer.deliver_attachments_added(attachments[:files]) if attachments.present? && attachments[:files].present? && Setting.notified_events.include?('document_added')
     redirect_to :action => 'show', :id => @document
   end
-
-private
-  def find_project
-    @project = Project.find(params[:project_id])
-  rescue ActiveRecord::RecordNotFound
-    render_404
-  end
 end
