@@ -1,27 +1,23 @@
-# redMine - project management software
-# Copyright (C) 2006-2008  Jean-Philippe Lang
+# Redmine - project management software
+# Copyright (C) 2006-2011  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
 # as published by the Free Software Foundation; either version 2
 # of the License, or (at your option) any later version.
-# 
+#
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
-# 
+#
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 class MailHandlerController < ActionController::Base
   before_filter :check_credential
-  
-  verify :method => :post,
-         :only => :index,
-         :render => { :nothing => true, :status => 405 }
-         
+
   # Submits an incoming email to MailHandler
   def index
     options = params.dup
@@ -32,9 +28,9 @@ class MailHandlerController < ActionController::Base
       render :nothing => true, :status => :unprocessable_entity
     end
   end
-  
+
   private
-  
+
   def check_credential
     User.current = nil
     unless Setting.mail_handler_api_enabled? && params[:key].to_s == Setting.mail_handler_api_key
