@@ -1,16 +1,16 @@
-# redMine - project management software
-# Copyright (C) 2006  Jean-Philippe Lang
+# Redmine - project management software
+# Copyright (C) 2006-2011  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
 # as published by the Free Software Foundation; either version 2
 # of the License, or (at your option) any later version.
-# 
+#
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
-# 
+#
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
@@ -24,8 +24,8 @@ class ReportsController < ApplicationController
     @versions = @project.shared_versions.sort
     @priorities = IssuePriority.all
     @categories = @project.issue_categories
-    @assignees = @project.members.collect { |m| m.user }.sort
-    @authors = @project.members.collect { |m| m.user }.sort
+    @assignees = (Setting.issue_group_assignment? ? @project.principals : @project.users).sort
+    @authors = @project.users.sort
     @subprojects = @project.descendants.visible
 
     @issues_by_tracker = Issue.by_tracker(@project)
@@ -37,7 +37,7 @@ class ReportsController < ApplicationController
     @issues_by_subproject = Issue.by_subproject(@project) || []
 
     render :template => "reports/issue_report"
-  end  
+  end
 
   def issue_report_details
     case params[:detail]
@@ -63,12 +63,12 @@ class ReportsController < ApplicationController
       @report_title = l(:field_category)
     when "assigned_to"
       @field = "assigned_to_id"
-      @rows = @project.members.collect { |m| m.user }.sort
+      @rows = (Setting.issue_group_assignment? ? @project.principals : @project.users).sort
       @data = Issue.by_assigned_to(@project)
       @report_title = l(:field_assigned_to)
     when "author"
       @field = "author_id"
-      @rows = @project.members.collect { |m| m.user }.sort
+      @rows = @project.users.sort
       @data = Issue.by_author(@project)
       @report_title = l(:field_author)
     when "subproject"
