@@ -17,6 +17,7 @@
 
 class SettingsController < ApplicationController
   layout 'admin'
+  menu_item :plugins, :only => :plugin
 
   before_filter :require_admin
 
@@ -51,12 +52,12 @@ class SettingsController < ApplicationController
   def plugin
     @plugin = Redmine::Plugin.find(params[:id])
     if request.post?
-      Setting["plugin_#{@plugin.id}"] = params[:settings]
+      Setting.send "plugin_#{@plugin.id}=", params[:settings]
       flash[:notice] = l(:notice_successful_update)
       redirect_to :action => 'plugin', :id => @plugin.id
     else
       @partial = @plugin.settings[:partial]
-      @settings = Setting["plugin_#{@plugin.id}"]
+      @settings = Setting.send "plugin_#{@plugin.id}"
     end
   rescue Redmine::PluginNotFound
     render_404
