@@ -5,12 +5,12 @@
 # modify it under the terms of the GNU General Public License
 # as published by the Free Software Foundation; either version 2
 # of the License, or (at your option) any later version.
-# 
+#
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
-# 
+#
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
@@ -19,11 +19,7 @@ class WatchersController < ApplicationController
   before_filter :find_project
   before_filter :require_login, :check_project_privacy, :only => [:watch, :unwatch]
   before_filter :authorize, :only => [:new, :destroy]
-  
-  verify :method => :post,
-         :only => [ :watch, :unwatch ],
-         :render => { :nothing => true, :status => :method_not_allowed }
-  
+
   def watch
     if @watched.respond_to?(:visible?) && !@watched.visible?(User.current)
       render_403
@@ -31,19 +27,35 @@ class WatchersController < ApplicationController
       set_watcher(User.current, true)
     end
   end
-  
+
   def unwatch
     set_watcher(User.current, false)
   end
-  
+
   def new
-    @watcher = Watcher.new(params[:watcher])
-    @watcher.watchable = @watched
-    @watcher.save if request.post?
+    respond_to do |format|
+      format.js do
+        render :update do |page|
+          page.replace_html 'ajax-modal', :partial => 'watchers/new', :locals => {:watched => @watched}
+          page << "showModal('ajax-modal', '400px');"
+          page << "$('ajax-modal').addClassName('new-watcher');"
+        end
+      end
+    end
+  end
+
+  def create
+    if params[:watcher].is_a?(Hash) && request.post?
+      user_ids = params[:watcher][:user_ids] || [params[:watcher][:user_id]]
+      user_ids.each do |user_id|
+        Watcher.create(:watchable => @watched, :user_id => user_id)
+      end
+    end
     respond_to do |format|
       format.html { redirect_to :back }
       format.js do
         render :update do |page|
+          page.replace_html 'ajax-modal', :partial => 'watchers/new', :locals => {:watched => @watched}
           page.replace_html 'watchers', :partial => 'watchers/watchers', :locals => {:watched => @watched}
         end
       end
@@ -51,7 +63,26 @@ class WatchersController < ApplicationController
   rescue ::ActionController::RedirectBackError
     render :text => 'Watcher added.', :layout => true
   end
-  
+
+  def append
+    if params[:watcher].is_a?(Hash)
+      user_ids = params[:watcher][:user_ids] || [params[:watcher][:user_id]]
+      users = User.active.find_all_by_id(user_ids)
+      respond_to do |format|
+        format.js do
+          render :update do |page|
+            users.each do |user|
+              page.select("#issue_watcher_user_ids_#{user.id}").each do |item|
+                page.remove item
+              end
+            end
+            page.insert_html :bottom, 'watchers_inputs', :text => watchers_checkboxes(nil, users, true)
+          end
+        end
+      end
+    end
+  end
+
   def destroy
     @watched.set_watcher(User.find(params[:user_id]), false) if request.post?
     respond_to do |format|
@@ -63,17 +94,29 @@ class WatchersController < ApplicationController
       end
     end
   end
-  
+
+  def autocomplete_for_user
+    @users = User.active.like(params[:q]).find(:all, :limit => 100)
+    if @watched
+      @users -= @watched.watcher_users
+    end
+    render :layout => false
+  end
+
 private
   def find_project
-    klass = Object.const_get(params[:object_type].camelcase)
-    return false unless klass.respond_to?('watched_by')
-    @watched = klass.find(params[:object_id])
-    @project = @watched.project
+    if params[:object_type] && params[:object_id]
+      klass = Object.const_get(params[:object_type].camelcase)
+      return false unless klass.respond_to?('watched_by')
+      @watched = klass.find(params[:object_id])
+      @project = @watched.project
+    elsif params[:project_id]
+      @project = Project.visible.find(params[:project_id])
+    end
   rescue
     render_404
   end
-  
+
   def set_watcher(user, watching)
     @watched.set_watcher(user, watching)
     respond_to do |format|
