@@ -18,4 +18,7 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 module AuthSourcesHelper
+  def auth_source_partial_name(auth_source)
+    "form_#{auth_source.class.name.underscore}"
+  end
 end
