@@ -15,8 +15,6 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
-require 'ar_condition'
-
 class Mailer < ActionMailer::Base
   layout 'mailer'
   helper :application
@@ -43,6 +41,7 @@ class Mailer < ActionMailer::Base
                     'Issue-Author' => issue.author.login
     redmine_headers 'Issue-Assignee' => issue.assigned_to.login if issue.assigned_to
     message_id issue
+    @author = issue.author
     recipients issue.recipients
     cc(issue.watcher_recipients - @recipients)
     subject "[#{issue.project.name} - #{issue.tracker.name} ##{issue.id}] (#{issue.status.name}) #{issue.subject}"
@@ -74,7 +73,7 @@ class Mailer < ActionMailer::Base
     subject s
     body :issue => issue,
          :journal => journal,
-         :issue_url => url_for(:controller => 'issues', :action => 'show', :id => issue)
+         :issue_url => url_for(:controller => 'issues', :action => 'show', :id => issue, :anchor => "change-#{journal.id}")
 
     render_multipart('issue_edit', body)
   end
@@ -85,7 +84,9 @@ class Mailer < ActionMailer::Base
     subject l(:mail_subject_reminder, :count => issues.size, :days => days)
     body :issues => issues,
          :days => days,
-         :issues_url => url_for(:controller => 'issues', :action => 'index', :set_filter => 1, :assigned_to_id => user.id, :sort => 'due_date:asc')
+         :issues_url => url_for(:controller => 'issues', :action => 'index',
+                                :set_filter => 1, :assigned_to_id => user.id,
+                                :sort => 'due_date:asc')
     render_multipart('reminder', body)
   end
 
@@ -97,6 +98,7 @@ class Mailer < ActionMailer::Base
   def document_added(document)
     redmine_headers 'Project' => document.project.identifier
     recipients document.recipients
+    @author = User.current
     subject "[#{document.project.name}] #{l(:label_document_new)}: #{document.title}"
     body :document => document,
          :document_url => url_for(:controller => 'documents', :action => 'show', :id => document)
@@ -112,6 +114,7 @@ class Mailer < ActionMailer::Base
     container = attachments.first.container
     added_to = ''
     added_to_url = ''
+    @author = attachments.first.author
     case container.class.name
     when 'Project'
       added_to_url = url_for(:controller => 'files', :action => 'index', :project_id => container)
@@ -141,6 +144,7 @@ class Mailer < ActionMailer::Base
   #   Mailer.deliver_news_added(news) => sends an email to the news' project recipients
   def news_added(news)
     redmine_headers 'Project' => news.project.identifier
+    @author = news.author
     message_id news
     recipients news.recipients
     subject "[#{news.project.name}] #{l(:label_news)}: #{news.title}"
@@ -157,6 +161,7 @@ class Mailer < ActionMailer::Base
   def news_comment_added(comment)
     news = comment.commented
     redmine_headers 'Project' => news.project.identifier
+    @author = comment.author
     message_id comment
     recipients news.recipients
     cc news.watcher_recipients
@@ -175,6 +180,7 @@ class Mailer < ActionMailer::Base
   def message_posted(message)
     redmine_headers 'Project' => message.project.identifier,
                     'Topic-Id' => (message.parent_id || message.id)
+    @author = message.author
     message_id message
     references message.parent unless message.parent.nil?
     recipients(message.recipients)
@@ -193,12 +199,15 @@ class Mailer < ActionMailer::Base
   def wiki_content_added(wiki_content)
     redmine_headers 'Project' => wiki_content.project.identifier,
                     'Wiki-Page-Id' => wiki_content.page.id
+    @author = wiki_content.author
     message_id wiki_content
     recipients wiki_content.recipients
     cc(wiki_content.page.wiki.watcher_recipients - recipients)
     subject "[#{wiki_content.project.name}] #{l(:mail_subject_wiki_content_added, :id => wiki_content.page.pretty_title)}"
     body :wiki_content => wiki_content,
-         :wiki_content_url => url_for(:controller => 'wiki', :action => 'show', :project_id => wiki_content.project, :id => wiki_content.page.title)
+         :wiki_content_url => url_for(:controller => 'wiki', :action => 'show',
+                                      :project_id => wiki_content.project,
+                                      :id => wiki_content.page.title)
     render_multipart('wiki_content_added', body)
   end
 
@@ -210,13 +219,18 @@ class Mailer < ActionMailer::Base
   def wiki_content_updated(wiki_content)
     redmine_headers 'Project' => wiki_content.project.identifier,
                     'Wiki-Page-Id' => wiki_content.page.id
+    @author = wiki_content.author
     message_id wiki_content
     recipients wiki_content.recipients
     cc(wiki_content.page.wiki.watcher_recipients + wiki_content.page.watcher_recipients - recipients)
     subject "[#{wiki_content.project.name}] #{l(:mail_subject_wiki_content_updated, :id => wiki_content.page.pretty_title)}"
     body :wiki_content => wiki_content,
-         :wiki_content_url => url_for(:controller => 'wiki', :action => 'show', :project_id => wiki_content.project, :id => wiki_content.page.title),
-         :wiki_diff_url => url_for(:controller => 'wiki', :action => 'diff', :project_id => wiki_content.project, :id => wiki_content.page.title, :version => wiki_content.version)
+         :wiki_content_url => url_for(:controller => 'wiki', :action => 'show',
+                                      :project_id => wiki_content.project,
+                                      :id => wiki_content.page.title),
+         :wiki_diff_url => url_for(:controller => 'wiki', :action => 'diff',
+                                   :project_id => wiki_content.project, :id => wiki_content.page.title,
+                                   :version => wiki_content.version)
     render_multipart('wiki_content_updated', body)
   end
 
@@ -245,7 +259,9 @@ class Mailer < ActionMailer::Base
     recipients User.active.find(:all, :conditions => {:admin => true}).collect { |u| u.mail }.compact
     subject l(:mail_subject_account_activation_request, Setting.app_title)
     body :user => user,
-         :url => url_for(:controller => 'users', :action => 'index', :status => User::STATUS_REGISTERED, :sort_key => 'created_on', :sort_order => 'desc')
+         :url => url_for(:controller => 'users', :action => 'index',
+                         :status => User::STATUS_REGISTERED,
+                         :sort_key => 'created_on', :sort_order => 'desc')
     render_multipart('account_activation_request', body)
   end
 
@@ -281,12 +297,12 @@ class Mailer < ActionMailer::Base
     render_multipart('register', body)
   end
 
-  def test(user)
+  def test_email(user)
     set_language_if_valid(user.language)
     recipients user.mail
     subject 'Redmine test'
     body :url => url_for(:controller => 'welcome')
-    render_multipart('test', body)
+    render_multipart('test_email', body)
   end
 
   # Overrides default deliver! method to prevent from sending an email
@@ -333,18 +349,17 @@ class Mailer < ActionMailer::Base
     tracker = options[:tracker] ? Tracker.find(options[:tracker]) : nil
     user_ids = options[:users]
 
-    s = ARCondition.new ["#{IssueStatus.table_name}.is_closed = ? AND #{Issue.table_name}.due_date <= ?", false, days.day.from_now.to_date]
-    s << "#{Issue.table_name}.assigned_to_id IS NOT NULL"
-    s << ["#{Issue.table_name}.assigned_to_id IN (?)", user_ids] if user_ids.present?
-    s << "#{Project.table_name}.status = #{Project::STATUS_ACTIVE}"
-    s << "#{Issue.table_name}.project_id = #{project.id}" if project
-    s << "#{Issue.table_name}.tracker_id = #{tracker.id}" if tracker
+    scope = Issue.open.scoped(:conditions => ["#{Issue.table_name}.assigned_to_id IS NOT NULL" +
+      " AND #{Project.table_name}.status = #{Project::STATUS_ACTIVE}" +
+      " AND #{Issue.table_name}.due_date <= ?", days.day.from_now.to_date]
+    )
+    scope = scope.scoped(:conditions => {:assigned_to_id => user_ids}) if user_ids.present?
+    scope = scope.scoped(:conditions => {:project_id => project.id}) if project
+    scope = scope.scoped(:conditions => {:tracker_id => tracker.id}) if tracker
 
-    issues_by_assignee = Issue.find(:all, :include => [:status, :assigned_to, :project, :tracker],
-                                          :conditions => s.conditions
-                                    ).group_by(&:assigned_to)
+    issues_by_assignee = scope.all(:include => [:status, :assigned_to, :project, :tracker]).group_by(&:assigned_to)
     issues_by_assignee.each do |assignee, issues|
-      deliver_reminder(assignee, issues, days) if assignee && assignee.active?
+      deliver_reminder(assignee, issues, days) if assignee.is_a?(User) && assignee.active?
     end
   end
 
@@ -357,6 +372,17 @@ class Mailer < ActionMailer::Base
     ActionMailer::Base.perform_deliveries = was_enabled
   end
 
+  # Sends emails synchronously in the given block
+  def self.with_synched_deliveries(&block)
+    saved_method = ActionMailer::Base.delivery_method
+    if m = saved_method.to_s.match(%r{^async_(.+)$})
+      ActionMailer::Base.delivery_method = m[1].to_sym
+    end
+    yield
+  ensure
+    ActionMailer::Base.delivery_method = saved_method
+  end
+
   private
   def initialize_defaults(method_name)
     super
@@ -368,23 +394,31 @@ class Mailer < ActionMailer::Base
     headers 'X-Mailer' => 'Redmine',
             'X-Redmine-Host' => Setting.host_name,
             'X-Redmine-Site' => Setting.app_title,
-            'Precedence' => 'bulk',
-            'Auto-Submitted' => 'auto-generated'
+            'X-Auto-Response-Suppress' => 'OOF',
+            'Auto-Submitted' => 'auto-generated',
+            'List-Id' => "<#{Setting.mail_from.to_s.gsub('@', '.')}>"
   end
 
   # Appends a Redmine header field (name is prepended with 'X-Redmine-')
   def redmine_headers(h)
-    h.each { |k,v| headers["X-Redmine-#{k}"] = v }
+    h.each { |k,v| headers["X-Redmine-#{k}"] = v.to_s }
   end
 
   # Overrides the create_mail method
   def create_mail
-    # Removes the current user from the recipients and cc
+    # Removes the author from the recipients and cc
     # if he doesn't want to receive notifications about what he does
-    @author ||= User.current
-    if @author.pref[:no_self_notified]
-      recipients.delete(@author.mail) if recipients
-      cc.delete(@author.mail) if cc
+    if @author && @author.logged? && @author.pref[:no_self_notified]
+      if recipients
+        recipients((recipients.is_a?(Array) ? recipients : [recipients]) - [@author.mail])
+      end
+      if cc
+        cc((cc.is_a?(Array) ? cc : [cc]) - [@author.mail])
+      end
+    end
+
+    if @author && @author.logged?
+      redmine_headers 'Sender' => @author.login
     end
 
     notified_users = [recipients, cc].flatten.compact.uniq
@@ -410,11 +444,16 @@ class Mailer < ActionMailer::Base
   def render_multipart(method_name, body)
     if Setting.plain_text_mail?
       content_type "text/plain"
-      body render(:file => "#{method_name}.text.plain.rhtml", :body => body, :layout => 'mailer.text.plain.erb')
+      body render(:file => "#{method_name}.text.erb",
+                  :body => body,
+                  :layout => 'mailer.text.erb')
     else
       content_type "multipart/alternative"
-      part :content_type => "text/plain", :body => render(:file => "#{method_name}.text.plain.rhtml", :body => body, :layout => 'mailer.text.plain.erb')
-      part :content_type => "text/html", :body => render_message("#{method_name}.text.html.rhtml", body)
+      part :content_type => "text/plain",
+           :body => render(:file => "#{method_name}.text.erb",
+                           :body => body, :layout => 'mailer.text.erb')
+      part :content_type => "text/html",
+           :body => render_message("#{method_name}.html.erb", body)
     end
   end
 
@@ -446,7 +485,7 @@ class Mailer < ActionMailer::Base
   end
 
   def mylogger
-    RAILS_DEFAULT_LOGGER
+    Rails.logger
   end
 end
 
