@@ -24,9 +24,17 @@ class Member < ActiveRecord::Base
 
   validates_presence_of :principal, :project
   validates_uniqueness_of :user_id, :scope => :project_id
+  validate :validate_role
 
+  before_destroy :set_issue_category_nil
   after_destroy :unwatch_from_permission_change
 
+  def role
+  end
+
+  def role=
+  end
+
   def name
     self.user.name
   end
@@ -75,7 +83,7 @@ class Member < ActiveRecord::Base
     end
   end
 
-  def before_destroy
+  def set_issue_category_nil
     if user
       # remove category based auto assignments for this member
       IssueCategory.update_all "assigned_to_id = NULL", ["project_id = ? AND assigned_to_id = ?", project.id, user.id]
@@ -91,7 +99,7 @@ class Member < ActiveRecord::Base
 
   protected
 
-  def validate
+  def validate_role
     errors.add_on_empty :role if member_roles.empty? && roles.empty?
   end
 
