@@ -21,14 +21,14 @@ require 'digest/sha1'
 class Repository::Cvs < Repository
   validates_presence_of :url, :root_url, :log_encoding
 
-  def self.human_attribute_name(attribute_key_name)
-    attr_name = attribute_key_name
+  def self.human_attribute_name(attribute_key_name, *args)
+    attr_name = attribute_key_name.to_s
     if attr_name == "root_url"
       attr_name = "cvsroot"
     elsif attr_name == "url"
       attr_name = "cvs_module"
     end
-    super(attr_name)
+    super(attr_name, *args)
   end
 
   def self.scm_adapter_class
@@ -181,10 +181,9 @@ class Repository::Cvs < Repository
       end
 
       # Renumber new changesets in chronological order
-      changesets.find(
-              :all,
+      Changeset.all(
               :order => 'committed_on ASC, id ASC',
-              :conditions => "revision LIKE 'tmp%'"
+              :conditions => ["repository_id = ? AND revision LIKE 'tmp%'", id]
            ).each do |changeset|
         changeset.update_attribute :revision, next_revision_number
       end
