@@ -17,13 +17,9 @@ config.action_mailer.perform_deliveries = true
 config.action_mailer.delivery_method = :test
 
 config.action_controller.session = { 
-  :session_key => "_test_session",
+  :key => "_test_session",
   :secret => "some secret phrase for the tests."
 }
 
 # Skip protect_from_forgery in requests http://m.onkey.org/2007/9/28/csrf-protection-for-your-existing-rails-application
 config.action_controller.allow_forgery_protection  = false
-
-config.gem "shoulda", :version => "~> 2.10.3"
-config.gem "edavis10-object_daddy", :lib => "object_daddy"
-config.gem "mocha"
