@@ -1,7 +1,7 @@
 # encoding: utf-8
 #
 # Redmine - project management software
-# Copyright (C) 2006-2011  Jean-Philippe Lang
+# Copyright (C) 2006-2012  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -76,6 +76,8 @@ module Redmine
           end
           SetCreator(Redmine::Info.app_name)
           SetFont(@font_for_content)
+          @outlines = []
+          @outlineRoot = nil
         end
 
         def SetFontStyle(style, size)
@@ -142,6 +144,245 @@ module Redmine
           SetX(-30)
           RDMCell(0, 5, PageNo().to_s + '/{nb}', 0, 0, 'C')
         end
+
+        def Bookmark(txt, level=0, y=0)
+          if (y == -1)
+            y = GetY()
+          end
+          @outlines << {:t => txt, :l => level, :p => PageNo(), :y => (@h - y)*@k}
+        end
+
+        def bookmark_title(txt)
+          txt = begin
+            utf16txt = Iconv.conv('UTF-16BE', 'UTF-8', txt)
+            hextxt = "<FEFF"  # FEFF is BOM
+            hextxt << utf16txt.unpack("C*").map {|x| sprintf("%02X",x) }.join
+            hextxt << ">"
+          rescue
+            txt
+          end || ''
+        end
+
+        def putbookmarks
+          nb=@outlines.size
+          return if (nb==0)
+          lru=[]
+          level=0
+          @outlines.each_with_index do |o, i|
+            if(o[:l]>0)
+              parent=lru[o[:l]-1]
+              #Set parent and last pointers
+              @outlines[i][:parent]=parent
+              @outlines[parent][:last]=i
+              if (o[:l]>level)
+                #Level increasing: set first pointer
+                @outlines[parent][:first]=i
+              end
+            else
+              @outlines[i][:parent]=nb
+            end
+            if (o[:l]<=level && i>0)
+              #Set prev and next pointers
+              prev=lru[o[:l]]
+              @outlines[prev][:next]=i
+              @outlines[i][:prev]=prev
+            end
+            lru[o[:l]]=i
+            level=o[:l]
+          end
+          #Outline items
+          n=self.n+1
+          @outlines.each_with_index do |o, i|
+            newobj()
+            out('<</Title '+bookmark_title(o[:t]))
+            out("/Parent #{n+o[:parent]} 0 R")
+            if (o[:prev])
+              out("/Prev #{n+o[:prev]} 0 R")
+            end
+            if (o[:next])
+              out("/Next #{n+o[:next]} 0 R")
+            end
+            if (o[:first])
+              out("/First #{n+o[:first]} 0 R")
+            end
+            if (o[:last])
+              out("/Last #{n+o[:last]} 0 R")
+            end
+            out("/Dest [%d 0 R /XYZ 0 %.2f null]" % [1+2*o[:p], o[:y]])
+            out('/Count 0>>')
+            out('endobj')
+          end
+          #Outline root
+          newobj()
+          @outlineRoot=self.n
+          out("<</Type /Outlines /First #{n} 0 R");
+          out("/Last #{n+lru[0]} 0 R>>");
+          out('endobj');
+        end
+
+        def putresources()
+          super
+          putbookmarks()
+        end
+
+        def putcatalog()
+          super
+          if(@outlines.size > 0)
+            out("/Outlines #{@outlineRoot} 0 R");
+            out('/PageMode /UseOutlines');
+          end
+        end
+      end
+
+      # fetch row values
+      def fetch_row_values(issue, query, level)
+        query.columns.collect do |column|
+          s = if column.is_a?(QueryCustomFieldColumn)
+            cv = issue.custom_field_values.detect {|v| v.custom_field_id == column.custom_field.id}
+            show_value(cv)
+          else
+            value = issue.send(column.name)
+            if column.name == :subject
+              value = "  " * level + value
+            end
+            if value.is_a?(Date)
+              format_date(value)
+            elsif value.is_a?(Time)
+              format_time(value)
+            else
+              value
+            end
+          end
+          s.to_s
+        end
+      end
+
+      # calculate columns width
+      def calc_col_width(issues, query, table_width, pdf)
+        # calculate statistics
+        #  by captions
+        pdf.SetFontStyle('B',8)
+        col_padding = pdf.GetStringWidth('OO')
+        col_width_min = query.columns.map {|v| pdf.GetStringWidth(v.caption) + col_padding}
+        col_width_max = Array.new(col_width_min)
+        col_width_avg = Array.new(col_width_min)
+        word_width_max = query.columns.map {|c|
+          n = 10
+          c.caption.split.each {|w|
+            x = pdf.GetStringWidth(w) + col_padding
+            n = x if n < x
+          }
+          n
+        }
+
+        #  by properties of issues
+        pdf.SetFontStyle('',8)
+        col_padding = pdf.GetStringWidth('OO')
+        k = 1
+        issue_list(issues) {|issue, level|
+          k += 1
+          values = fetch_row_values(issue, query, level)
+          values.each_with_index {|v,i|
+            n = pdf.GetStringWidth(v) + col_padding
+            col_width_max[i] = n if col_width_max[i] < n
+            col_width_min[i] = n if col_width_min[i] > n
+            col_width_avg[i] += n
+            v.split.each {|w|
+              x = pdf.GetStringWidth(w) + col_padding
+              word_width_max[i] = x if word_width_max[i] < x
+            }
+          }
+        }
+        col_width_avg.map! {|x| x / k}
+
+        # calculate columns width
+        ratio = table_width / col_width_avg.inject(0) {|s,w| s += w}
+        col_width = col_width_avg.map {|w| w * ratio}
+
+        # correct max word width if too many columns
+        ratio = table_width / word_width_max.inject(0) {|s,w| s += w}
+        word_width_max.map! {|v| v * ratio} if ratio < 1
+
+        # correct and lock width of some columns
+        done = 1
+        col_fix = []
+        col_width.each_with_index do |w,i|
+          if w > col_width_max[i]
+            col_width[i] = col_width_max[i]
+            col_fix[i] = 1
+            done = 0
+          elsif w < word_width_max[i]
+            col_width[i] = word_width_max[i]
+            col_fix[i] = 1
+            done = 0
+          else
+            col_fix[i] = 0
+          end
+        end
+
+        # iterate while need to correct and lock coluns width
+        while done == 0
+          # calculate free & locked columns width
+          done = 1
+          fix_col_width = 0
+          free_col_width = 0
+          col_width.each_with_index do |w,i|
+            if col_fix[i] == 1
+              fix_col_width += w
+            else
+              free_col_width += w
+            end
+          end
+
+          # calculate column normalizing ratio
+          if free_col_width == 0
+            ratio = table_width / col_width.inject(0) {|s,w| s += w}
+          else
+            ratio = (table_width - fix_col_width) / free_col_width
+          end
+
+          # correct columns width
+          col_width.each_with_index do |w,i|
+            if col_fix[i] == 0
+              col_width[i] = w * ratio
+
+              # check if column width less then max word width
+              if col_width[i] < word_width_max[i]
+                col_width[i] = word_width_max[i]
+                col_fix[i] = 1
+                done = 0
+              elsif col_width[i] > col_width_max[i]
+                col_width[i] = col_width_max[i]
+                col_fix[i] = 1
+                done = 0
+              end
+            end
+          end
+        end
+        col_width
+      end
+
+      def render_table_header(pdf, query, col_width, row_height, col_id_width, table_width)
+        # headers
+        pdf.SetFontStyle('B',8)
+        pdf.SetFillColor(230, 230, 230)
+
+        # render it background to find the max height used
+        base_x = pdf.GetX
+        base_y = pdf.GetY
+        max_height = issues_to_pdf_write_cells(pdf, query.columns, col_width, row_height, true)
+        pdf.Rect(base_x, base_y, table_width + col_id_width, max_height, 'FD');
+        pdf.SetXY(base_x, base_y);
+
+        # write the cells on page
+        pdf.RDMCell(col_id_width, row_height, "#", "T", 0, 'C', 1)
+        issues_to_pdf_write_cells(pdf, query.columns, col_width, row_height, true)
+        issues_to_pdf_draw_borders(pdf, base_x, base_y, base_y + max_height, col_id_width, col_width)
+        pdf.SetY(base_y + max_height);
+
+        # rows
+        pdf.SetFontStyle('',8)
+        pdf.SetFillColor(255, 255, 255)
       end
 
       # Returns a PDF string of a list of issues
@@ -161,76 +402,36 @@ module Redmine
         right_margin  = 10
         bottom_margin = 20
         col_id_width  = 10
-        row_height    = 5
+        row_height    = 4
 
         # column widths
         table_width = page_width - right_margin - 10  # fixed left margin
         col_width = []
         unless query.columns.empty?
-          col_width = query.columns.collect do |c|
-            (c.name == :subject || (c.is_a?(QueryCustomFieldColumn) &&
-              ['string', 'text'].include?(c.custom_field.field_format))) ? 4.0 : 1.0
-          end
-          ratio = (table_width - col_id_width) / col_width.inject(0) {|s,w| s += w}
-          col_width = col_width.collect {|w| w * ratio}
+          col_width = calc_col_width(issues, query, table_width - col_id_width, pdf)
+          table_width = col_width.inject(0) {|s,v| s += v}
         end
 
         # title
         pdf.SetFontStyle('B',11)
         pdf.RDMCell(190,10, title)
         pdf.Ln
-
-        # headers
-        pdf.SetFontStyle('B',8)
-        pdf.SetFillColor(230, 230, 230)
-
-        # render it background to find the max height used
-        base_x = pdf.GetX
-        base_y = pdf.GetY
-        max_height = issues_to_pdf_write_cells(pdf, query.columns, col_width, row_height, true)
-        pdf.Rect(base_x, base_y, table_width, max_height, 'FD');
-        pdf.SetXY(base_x, base_y);
-
-        # write the cells on page
-        pdf.RDMCell(col_id_width, row_height, "#", "T", 0, 'C', 1)
-        issues_to_pdf_write_cells(pdf, query.columns, col_width, row_height, true)
-        issues_to_pdf_draw_borders(pdf, base_x, base_y, base_y + max_height, col_id_width, col_width)
-        pdf.SetY(base_y + max_height);
-
-        # rows
-        pdf.SetFontStyle('',8)
-        pdf.SetFillColor(255, 255, 255)
+        render_table_header(pdf, query, col_width, row_height, col_id_width, table_width)
         previous_group = false
         issue_list(issues) do |issue, level|
           if query.grouped? &&
                (group = query.group_by_column.value(issue)) != previous_group
-            pdf.SetFontStyle('B',9)
-            pdf.RDMCell(277, row_height,
-              (group.blank? ? 'None' : group.to_s) + " (#{query.issue_count_by_group[group]})",
-              1, 1, 'L')
+            pdf.SetFontStyle('B',10)
+            group_label = group.blank? ? 'None' : group.to_s
+            group_label << " (#{query.issue_count_by_group[group]})"
+            pdf.Bookmark group_label, 0, -1
+            pdf.RDMCell(table_width + col_id_width, row_height * 2, group_label, 1, 1, 'L')
             pdf.SetFontStyle('',8)
             previous_group = group
           end
-          # fetch all the row values
-          col_values = query.columns.collect do |column|
-            s = if column.is_a?(QueryCustomFieldColumn)
-              cv = issue.custom_values.detect {|v| v.custom_field_id == column.custom_field.id}
-              show_value(cv)
-            else
-              value = issue.send(column.name)
-              if column.name == :subject
-                value = "  " * level + value
-              end
-              if value.is_a?(Date)
-                format_date(value)
-              elsif value.is_a?(Time)
-                format_time(value)
-              else
-                value
-              end
-            end
-            s.to_s
-          end
+
+          # fetch row values
+          col_values = fetch_row_values(issue, query, level)
 
           # render it off-page to find the max height used
           base_x = pdf.GetX
@@ -243,6 +444,7 @@ module Redmine
           space_left = page_height - base_y - bottom_margin
           if max_height > space_left
             pdf.AddPage("L")
+            render_table_header(pdf, query, col_width, row_height, col_id_width, table_width)
             base_x = pdf.GetX
             base_y = pdf.GetY
           end
@@ -381,7 +583,7 @@ module Redmine
         unless issue.leaf?
           # for CJK
           truncate_length = ( l(:general_pdf_encoding).upcase == "UTF-8" ? 90 : 65 )
-  
+
           pdf.SetFontStyle('B',9)
           pdf.RDMCell(35+155,5, l(:label_subtask_plural) + ":", "LTR")
           pdf.Ln
@@ -401,7 +603,7 @@ module Redmine
         unless relations.empty?
           # for CJK
           truncate_length = ( l(:general_pdf_encoding).upcase == "UTF-8" ? 80 : 60 )
-  
+
           pdf.SetFontStyle('B',9)
           pdf.RDMCell(35+155,5, l(:label_related_issues) + ":", "LTR")
           pdf.Ln
@@ -464,8 +666,8 @@ module Redmine
              " - " + journal.user.name)
           pdf.Ln
           pdf.SetFontStyle('I',8)
-          for detail in journal.details
-            pdf.RDMMultiCell(190,5, "- " + show_detail(detail, true))
+          details_to_strings(journal.details, true).each do |string|
+            pdf.RDMMultiCell(190,5, "- " + string)
           end
           if journal.notes?
             pdf.Ln unless journal.details.empty?
@@ -492,8 +694,25 @@ module Redmine
         pdf.Output
       end
 
+      # Returns a PDF string of a set of wiki pages
+      def wiki_pages_to_pdf(pages, project)
+        pdf = ITCPDF.new(current_language)
+        pdf.SetTitle(project.name)
+        pdf.alias_nb_pages
+        pdf.footer_date = format_date(Date.today)
+        pdf.AddPage
+        pdf.SetFontStyle('B',11)
+        pdf.RDMMultiCell(190,5, project.name)
+        pdf.Ln
+        # Set resize image scale
+        pdf.SetImageScale(1.6)
+        pdf.SetFontStyle('',9)
+        write_page_hierarchy(pdf, pages.group_by(&:parent_id))
+        pdf.Output
+      end
+
       # Returns a PDF string of a single wiki page
-      def wiki_to_pdf(page, project)
+      def wiki_page_to_pdf(page, project)
         pdf = ITCPDF.new(current_language)
         pdf.SetTitle("#{project} - #{page.title}")
         pdf.alias_nb_pages
@@ -506,8 +725,28 @@ module Redmine
         # Set resize image scale
         pdf.SetImageScale(1.6)
         pdf.SetFontStyle('',9)
+        write_wiki_page(pdf, page)
+        pdf.Output
+      end
+
+      def write_page_hierarchy(pdf, pages, node=nil, level=0)
+        if pages[node]
+          pages[node].each do |page|
+            if @new_page
+              pdf.AddPage
+            else
+              @new_page = true
+            end
+            pdf.Bookmark page.title, level
+            write_wiki_page(pdf, page)
+            write_page_hierarchy(pdf, pages, page.id, level + 1) if pages[page.id]
+          end
+        end
+      end
+
+      def write_wiki_page(pdf, page)
         pdf.RDMwriteHTMLCell(190,5,0,0,
-              page.content.text.to_s, page.attachments, "TLRB")
+              page.content.text.to_s, page.attachments, 0)
         if page.attachments.any?
           pdf.Ln
           pdf.SetFontStyle('B',9)
@@ -522,7 +761,6 @@ module Redmine
             pdf.Ln
           end
         end
-        pdf.Output
       end
 
       class RDMPdfEncoding
