@@ -35,7 +35,7 @@ module Redmine
             logger.debug "#{pop_session.mails.size} email(s) to process..." if logger && logger.debug?
             pop_session.each_mail do |msg|
               message = msg.pop
-              message_id = (message =~ /^Message-ID: (.*)/ ? $1 : '').strip
+              message_id = (message =~ /^Message-I[dD]: (.*)/ ? $1 : '').strip
               if MailHandler.receive(message, options)
                 msg.delete
                 logger.debug "--> Message #{message_id} processed and deleted from the server" if logger && logger.debug?
