@@ -10,7 +10,6 @@ class ActivitiesControllerTest < ActionController::TestCase
            :groups_users,
            :enabled_modules,
            :workflows,
-           :auth_sources,
            :journals, :journal_details
 
 
@@ -97,9 +96,39 @@ class ActivitiesControllerTest < ActionController::TestCase
   end
 
   def test_index_atom_feed
-    get :index, :format => 'atom'
+    get :index, :format => 'atom', :with_subprojects => 0
     assert_response :success
-    assert_template 'common/feed.atom'
+    assert_template 'common/feed'
+
+    assert_tag :tag => 'link', :parent =>  {:tag => 'feed', :parent => nil },
+        :attributes => {:rel => 'self', :href => 'http://test.host/activity.atom?with_subprojects=0'}
+    assert_tag :tag => 'link', :parent =>  {:tag => 'feed', :parent => nil },
+        :attributes => {:rel => 'alternate', :href => 'http://test.host/activity?with_subprojects=0'}
+
+    assert_tag :tag => 'entry', :child => {
+      :tag => 'link',
+      :attributes => {:href => 'http://test.host/issues/11'}}
+  end
+
+  def test_index_atom_feed_with_explicit_selection
+    get :index, :format => 'atom', :with_subprojects => 0,
+      :show_changesets => 1,
+      :show_documents => 1,
+      :show_files => 1,
+      :show_issues => 1,
+      :show_messages => 1,
+      :show_news => 1,
+      :show_time_entries => 1,
+      :show_wiki_edits => 1
+
+    assert_response :success
+    assert_template 'common/feed'
+
+    assert_tag :tag => 'link', :parent =>  {:tag => 'feed', :parent => nil },
+        :attributes => {:rel => 'self', :href => 'http://test.host/activity.atom?show_changesets=1&amp;show_documents=1&amp;show_files=1&amp;show_issues=1&amp;show_messages=1&amp;show_news=1&amp;show_time_entries=1&amp;show_wiki_edits=1&amp;with_subprojects=0'}
+    assert_tag :tag => 'link', :parent => {:tag => 'feed', :parent => nil },
+        :attributes => {:rel => 'alternate', :href => 'http://test.host/activity?show_changesets=1&amp;show_documents=1&amp;show_files=1&amp;show_issues=1&amp;show_messages=1&amp;show_news=1&amp;show_time_entries=1&amp;show_wiki_edits=1&amp;with_subprojects=0'}
+
     assert_tag :tag => 'entry', :child => {
       :tag => 'link',
       :attributes => {:href => 'http://test.host/issues/11'}}
@@ -108,7 +137,7 @@ class ActivitiesControllerTest < ActionController::TestCase
   def test_index_atom_feed_with_one_item_type
     get :index, :format => 'atom', :show_issues => '1'
     assert_response :success
-    assert_template 'common/feed.atom'
+    assert_template 'common/feed'
     assert_tag :tag => 'title', :content => /Issues/
   end
 end
