@@ -104,9 +104,9 @@ class MessagesControllerTest < ActionController::TestCase
     assert_equal 1, message.board_id
 
     mail = ActionMailer::Base.deliveries.last
-    assert_kind_of TMail::Mail, mail
+    assert_not_nil mail
     assert_equal "[#{message.board.project.name} - #{message.board.name} - msg#{message.root.id}] Test created message", mail.subject
-    assert mail.body.include?('Message body')
+    assert_mail_body_match 'Message body', mail
     # author
     assert mail.bcc.include?('jsmith@somenet.foo')
     # project member
@@ -131,6 +131,30 @@ class MessagesControllerTest < ActionController::TestCase
     assert_equal 'New body', message.content
   end
 
+  def test_post_edit_sticky_and_locked
+    @request.session[:user_id] = 2
+    post :edit, :board_id => 1, :id => 1,
+                :message => { :subject => 'New subject',
+                              :content => 'New body',
+                              :locked => '1',
+                              :sticky => '1'}
+    assert_redirected_to '/boards/1/topics/1'
+    message = Message.find(1)
+    assert_equal true, message.sticky?
+    assert_equal true, message.locked?
+  end
+
+  def test_post_edit_should_allow_to_change_board
+    @request.session[:user_id] = 2
+    post :edit, :board_id => 1, :id => 1,
+                :message => { :subject => 'New subject',
+                              :content => 'New body',
+                              :board_id => 2}
+    assert_redirected_to '/boards/2/topics/1'
+    message = Message.find(1)
+    assert_equal Board.find(2), message.board
+  end
+
   def test_reply
     @request.session[:user_id] = 2
     post :reply, :board_id => 1, :id => 1, :reply => { :content => 'This is a test reply', :subject => 'Test reply' }
@@ -141,15 +165,45 @@ class MessagesControllerTest < ActionController::TestCase
 
   def test_destroy_topic
     @request.session[:user_id] = 2
-    post :destroy, :board_id => 1, :id => 1
+    assert_difference 'Message.count', -3 do
+      post :destroy, :board_id => 1, :id => 1
+    end
     assert_redirected_to '/projects/ecookbook/boards/1'
     assert_nil Message.find_by_id(1)
   end
 
+  def test_destroy_reply
+    @request.session[:user_id] = 2
+    assert_difference 'Message.count', -1 do
+      post :destroy, :board_id => 1, :id => 2
+    end
+    assert_redirected_to '/boards/1/topics/1?r=2'
+    assert_nil Message.find_by_id(2)
+  end
+
   def test_quote
     @request.session[:user_id] = 2
     xhr :get, :quote, :board_id => 1, :id => 3
     assert_response :success
     assert_select_rjs :show, 'reply'
   end
+
+  def test_preview_new
+    @request.session[:user_id] = 2
+    post :preview,
+      :board_id => 1,
+      :message => {:subject => "", :content => "Previewed text"}
+    assert_response :success
+    assert_template 'common/_preview'
+  end
+
+  def test_preview_edit
+    @request.session[:user_id] = 2
+    post :preview,
+      :id => 4,
+      :board_id => 1,
+      :message => {:subject => "", :content => "Previewed text"}
+    assert_response :success
+    assert_template 'common/_preview'
+  end
 end
