@@ -55,7 +55,7 @@ class ProjectsControllerTest < ActionController::TestCase
   def test_index_atom
     get :index, :format => 'atom'
     assert_response :success
-    assert_template 'common/feed.atom'
+    assert_template 'common/feed'
     assert_select 'feed>title', :text => 'Redmine: Latest projects'
     assert_select 'feed>entry', :count => Project.count(:conditions => Project.visible_condition(User.current))
   end
@@ -311,12 +311,6 @@ class ProjectsControllerTest < ActionController::TestCase
     end
   end
 
-  def test_create_should_not_accept_get
-    @request.session[:user_id] = 1
-    get :create
-    assert_response :method_not_allowed
-  end
-
   def test_show_by_id
     get :show, :id => 1
     assert_response :success
@@ -395,6 +389,14 @@ class ProjectsControllerTest < ActionController::TestCase
     assert_equal 'Test changed name', project.name
   end
 
+  def test_update_with_failure
+    @request.session[:user_id] = 2 # manager
+    post :update, :id => 1, :project => {:name => ''}
+    assert_response :success
+    assert_template 'settings'
+    assert_error_tag :content => /name can't be blank/i
+  end
+
   def test_modules
     @request.session[:user_id] = 2
     Project.find(1).enabled_module_names = ['issue_tracking', 'news']
@@ -404,23 +406,17 @@ class ProjectsControllerTest < ActionController::TestCase
     assert_equal ['documents', 'issue_tracking', 'repository'], Project.find(1).enabled_module_names.sort
   end
 
-  def test_modules_should_not_allow_get
-    @request.session[:user_id] = 1
-    get :modules, :id => 1
-    assert_response :method_not_allowed
-  end
-
-  def test_get_destroy
+  def test_destroy_without_confirmation
     @request.session[:user_id] = 1 # admin
-    get :destroy, :id => 1
+    delete :destroy, :id => 1
     assert_response :success
     assert_template 'destroy'
     assert_not_nil Project.find_by_id(1)
   end
 
-  def test_post_destroy
+  def test_destroy
     @request.session[:user_id] = 1 # admin
-    post :destroy, :id => 1, :confirm => 1
+    delete :destroy, :id => 1, :confirm => 1
     assert_redirected_to '/admin/projects'
     assert_nil Project.find_by_id(1)
   end
@@ -432,6 +428,14 @@ class ProjectsControllerTest < ActionController::TestCase
     assert !Project.find(1).active?
   end
 
+  def test_archive_with_failure
+    @request.session[:user_id] = 1
+    Project.any_instance.stubs(:archive).returns(false)
+    post :archive, :id => 1
+    assert_redirected_to '/admin/projects'
+    assert_match /project cannot be archived/i, flash[:error]
+  end
+
   def test_unarchive
     @request.session[:user_id] = 1 # admin
     Project.find(1).archive
@@ -468,13 +472,6 @@ class ProjectsControllerTest < ActionController::TestCase
       :attributes => {:name => 'project[enabled_module_names][]', :value => 'issue_tracking'}
   end
 
-  def test_get_copy_without_project
-    @request.session[:user_id] = 1 # admin
-    get :copy
-    assert_response :redirect
-    assert_redirected_to :controller => 'admin', :action => 'projects'
-  end
-
   def test_post_copy_should_copy_requested_items
     @request.session[:user_id] = 1 # admin
     CustomField.delete_all
