@@ -16,12 +16,10 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 require File.expand_path('../../test_helper', __FILE__)
-require 'repositories_controller'
-
-# Re-raise errors caught by the controller.
-class RepositoriesController; def rescue_action(e) raise e end; end
 
 class RepositoriesFilesystemControllerTest < ActionController::TestCase
+  tests RepositoriesController
+
   fixtures :projects, :users, :roles, :members, :member_roles,
            :repositories, :enabled_modules
 
@@ -31,9 +29,6 @@ class RepositoriesFilesystemControllerTest < ActionController::TestCase
   def setup
     @ruby19_non_utf8_pass =
         (RUBY_VERSION >= '1.9' && Encoding.default_external.to_s != 'UTF-8')
-    @controller = RepositoriesController.new
-    @request    = ActionController::TestRequest.new
-    @response   = ActionController::TestResponse.new
     User.current = nil
     Setting.enabled_scm << 'Filesystem' unless Setting.enabled_scm.include?('Filesystem')
     @project = Project.find(PRJ_ID)
@@ -46,6 +41,16 @@ class RepositoriesFilesystemControllerTest < ActionController::TestCase
   end
 
   if File.directory?(REPOSITORY_PATH)
+    def test_get_new
+      @request.session[:user_id] = 1
+      @project.repository.destroy
+      get :new, :project_id => 'subproject1', :repository_scm => 'Filesystem'
+      assert_response :success
+      assert_template 'new'
+      assert_kind_of Repository::Filesystem, assigns(:repository)
+      assert assigns(:repository).new_record?
+    end
+
     def test_browse_root
       @repository.fetch_changesets
       @repository.reload
@@ -56,10 +61,14 @@ class RepositoriesFilesystemControllerTest < ActionController::TestCase
       assert assigns(:entries).size > 0
       assert_not_nil assigns(:changesets)
       assert assigns(:changesets).size == 0
+
+      assert_no_tag 'input', :attributes => {:name => 'rev'}
+      assert_no_tag 'a', :content => 'Statistics'
+      assert_no_tag 'a', :content => 'Atom'
     end
 
     def test_show_no_extension
-      get :entry, :id => PRJ_ID, :path => ['test']
+      get :entry, :id => PRJ_ID, :path => repository_path_hash(['test'])[:param]
       assert_response :success
       assert_template 'entry'
       assert_tag :tag => 'th',
@@ -69,14 +78,16 @@ class RepositoriesFilesystemControllerTest < ActionController::TestCase
     end
 
     def test_entry_download_no_extension
-      get :entry, :id => PRJ_ID, :path => ['test'], :format => 'raw'
+      get :entry, :id => PRJ_ID, :path => repository_path_hash(['test'])[:param],
+          :format => 'raw'
       assert_response :success
       assert_equal 'application/octet-stream', @response.content_type
     end
 
     def test_show_non_ascii_contents
       with_settings :repositories_encodings => 'UTF-8,EUC-JP' do
-        get :entry, :id => PRJ_ID, :path => ['japanese', 'euc-jp.txt']
+        get :entry, :id => PRJ_ID,
+            :path => repository_path_hash(['japanese', 'euc-jp.txt'])[:param]
         assert_response :success
         assert_template 'entry'
         assert_tag :tag => 'th',
@@ -100,7 +111,8 @@ class RepositoriesFilesystemControllerTest < ActionController::TestCase
 
     def test_show_utf16
       with_settings :repositories_encodings => 'UTF-16' do
-        get :entry, :id => PRJ_ID, :path => ['japanese', 'utf-16.txt']
+        get :entry, :id => PRJ_ID,
+            :path => repository_path_hash(['japanese', 'utf-16.txt'])[:param]
         assert_response :success
         assert_tag :tag => 'th',
                    :content => '2',
@@ -111,7 +123,8 @@ class RepositoriesFilesystemControllerTest < ActionController::TestCase
 
     def test_show_text_file_should_send_if_too_big
       with_settings :file_max_size_displayed => 1 do
-        get :entry, :id => PRJ_ID, :path => ['japanese', 'big-file.txt']
+        get :entry, :id => PRJ_ID,
+            :path => repository_path_hash(['japanese', 'big-file.txt'])[:param]
         assert_response :success
         assert_equal 'text/plain', @response.content_type
       end
@@ -120,7 +133,9 @@ class RepositoriesFilesystemControllerTest < ActionController::TestCase
     def test_destroy_valid_repository
       @request.session[:user_id] = 1 # admin
 
-      get :destroy, :id => PRJ_ID
+      assert_difference 'Repository.count', -1 do
+        delete :destroy, :id => @repository.id
+      end
       assert_response 302
       @project.reload
       assert_nil @project.repository
@@ -128,20 +143,16 @@ class RepositoriesFilesystemControllerTest < ActionController::TestCase
 
     def test_destroy_invalid_repository
       @request.session[:user_id] = 1 # admin
-
-      get :destroy, :id => PRJ_ID
-      assert_response 302
-      @project.reload
-      assert_nil @project.repository
-
-      @repository = Repository::Filesystem.create(
-                      :project       => Project.find(PRJ_ID),
+      @project.repository.destroy
+      @repository = Repository::Filesystem.create!(
+                      :project       => @project,
                       :url           => "/invalid",
                       :path_encoding => ''
                       )
-      assert @repository
 
-      get :destroy, :id => PRJ_ID
+      assert_difference 'Repository.count', -1 do
+        delete :destroy, :id => @repository.id
+      end
       assert_response 302
       @project.reload
       assert_nil @project.repository
