@@ -67,18 +67,40 @@ class SysControllerTest < ActionController::TestCase
     assert_no_tag 'extra_info'
   end
 
+  def test_create_already_existing
+    post :create_project_repository, :id => 1,
+      :vendor => 'Subversion',
+      :repository => { :url => 'file:///create/project/repository/subproject2'}
+
+    assert_response :conflict
+  end
+
+  def test_create_with_failure
+    post :create_project_repository, :id => 4,
+      :vendor => 'Subversion',
+      :repository => { :url => 'invalid url'}
+
+    assert_response :unprocessable_entity
+  end
+
   def test_fetch_changesets
-    Repository::Subversion.any_instance.expects(:fetch_changesets).returns(true)
+    Repository::Subversion.any_instance.expects(:fetch_changesets).twice.returns(true)
     get :fetch_changesets
     assert_response :success
   end
 
-  def test_fetch_changesets_one_project
-    Repository::Subversion.any_instance.expects(:fetch_changesets).returns(true)
+  def test_fetch_changesets_one_project_by_identifier
+    Repository::Subversion.any_instance.expects(:fetch_changesets).once.returns(true)
     get :fetch_changesets, :id => 'ecookbook'
     assert_response :success
   end
 
+  def test_fetch_changesets_one_project_by_id
+    Repository::Subversion.any_instance.expects(:fetch_changesets).once.returns(true)
+    get :fetch_changesets, :id => '1'
+    assert_response :success
+  end
+
   def test_fetch_changesets_unknown_project
     get :fetch_changesets, :id => 'unknown'
     assert_response 404
