@@ -1,7 +1,14 @@
 require File.expand_path('../../../test_helper', __FILE__)
 
 class ApiTest::HttpBasicLoginWithApiTokenTest < ActionController::IntegrationTest
-  fixtures :all
+  fixtures :projects, :trackers, :issue_statuses, :issues,
+           :enumerations, :users, :issue_categories,
+           :projects_trackers,
+           :roles,
+           :member_roles,
+           :members,
+           :enabled_modules,
+           :workflows
 
   def setup
     Setting.rest_api_enabled = '1'
@@ -12,7 +19,7 @@ class ApiTest::HttpBasicLoginWithApiTokenTest < ActionController::IntegrationTes
     Setting.rest_api_enabled = '0'
     Setting.login_required = '0'
   end
-  
+
   # Using the NewsController because it's a simple API.
   context "get /news" do
 
