@@ -5,12 +5,12 @@
 # modify it under the terms of the GNU General Public License
 # as published by the Free Software Foundation; either version 2
 # of the License, or (at your option) any later version.
-# 
+#
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
-# 
+#
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
@@ -18,7 +18,7 @@
 require File.expand_path('../../test_helper', __FILE__)
 
 class IssuesTest < ActionController::IntegrationTest
-  fixtures :projects, 
+  fixtures :projects,
            :users,
            :roles,
            :members,
@@ -38,13 +38,13 @@ class IssuesTest < ActionController::IntegrationTest
     get 'projects/1/issues/new', :tracker_id => '1'
     assert_response :success
     assert_template 'issues/new'
-    
+
     post 'projects/1/issues', :tracker_id => "1",
-                                 :issue => { :start_date => "2006-12-26", 
-                                             :priority_id => "4", 
-                                             :subject => "new test issue", 
-                                             :category_id => "", 
-                                             :description => "new issue", 
+                                 :issue => { :start_date => "2006-12-26",
+                                             :priority_id => "4",
+                                             :subject => "new test issue",
+                                             :category_id => "",
+                                             :description => "new issue",
                                              :done_ratio => "0",
                                              :due_date => "",
                                              :assigned_to_id => "" },
@@ -58,12 +58,21 @@ class IssuesTest < ActionController::IntegrationTest
     follow_redirect!
     assert_equal issue, assigns(:issue)
 
-    # check issue attributes    
+    # check issue attributes
     assert_equal 'jsmith', issue.author.login
     assert_equal 1, issue.project.id
     assert_equal 1, issue.status.id
   end
 
+  def test_update_issue_form
+    log_user('jsmith', 'jsmith')
+    post 'projects/ecookbook/issues/new', :issue => { :tracker_id => "2"}
+    assert_response :success
+    assert_tag 'select',
+      :attributes => {:name => 'issue[tracker_id]'},
+      :child => {:tag => 'option', :attributes => {:value => '2', :selected => 'selected'}}
+  end
+
   # add then remove 2 attachments to an issue
   def test_issue_attachments
     log_user('jsmith', 'jsmith')
@@ -73,7 +82,7 @@ class IssuesTest < ActionController::IntegrationTest
          :notes => 'Some notes',
          :attachments => {'1' => {'file' => uploaded_test_file('testfile.txt', 'text/plain'), 'description' => 'This is an attachment'}}
     assert_redirected_to "/issues/1"
-    
+
     # make sure attachment was saved
     attachment = Issue.find(1).attachments.find_by_filename("testfile.txt")
     assert_kind_of Attachment, attachment
@@ -83,56 +92,56 @@ class IssuesTest < ActionController::IntegrationTest
     #assert_equal file_data_1.length, attachment.filesize
     # verify that the attachment was written to disk
     assert File.exist?(attachment.diskfile)
-    
+
     # remove the attachments
     Issue.find(1).attachments.each(&:destroy)
     assert_equal 0, Issue.find(1).attachments.length
   end
-  
-  def test_other_formats_links_on_get_index
+
+  def test_other_formats_links_on_index
     get '/projects/ecookbook/issues'
-    
+
     %w(Atom PDF CSV).each do |format|
       assert_tag :a, :content => format,
                      :attributes => { :href => "/projects/ecookbook/issues.#{format.downcase}",
                                       :rel => 'nofollow' }
     end
   end
-  
-  def test_other_formats_links_on_post_index_without_project_id_in_url
-    post '/issues', :project_id => 'ecookbook'
-    
+
+  def test_other_formats_links_on_index_without_project_id_in_url
+    get '/issues', :project_id => 'ecookbook'
+
     %w(Atom PDF CSV).each do |format|
       assert_tag :a, :content => format,
                      :attributes => { :href => "/projects/ecookbook/issues.#{format.downcase}",
                                       :rel => 'nofollow' }
     end
   end
-  
-  def test_pagination_links_on_get_index
+
+  def test_pagination_links_on_index
     Setting.per_page_options = '2'
     get '/projects/ecookbook/issues'
-    
+
     assert_tag :a, :content => '2',
                    :attributes => { :href => '/projects/ecookbook/issues?page=2' }
-    
+
   end
-  
-  def test_pagination_links_on_post_index_without_project_id_in_url
+
+  def test_pagination_links_on_index_without_project_id_in_url
     Setting.per_page_options = '2'
-    post '/issues', :project_id => 'ecookbook'
-    
+    get '/issues', :project_id => 'ecookbook'
+
     assert_tag :a, :content => '2',
                    :attributes => { :href => '/projects/ecookbook/issues?page=2' }
-    
+
   end
-  
+
   def test_issue_with_user_custom_field
     @field = IssueCustomField.create!(:name => 'Tester', :field_format => 'user', :is_for_all => true, :trackers => Tracker.all)
     Role.anonymous.add_permission! :add_issues, :edit_issues
     users = Project.find(1).users
     tester = users.first
-    
+
     # Issue form
     get '/projects/ecookbook/issues/new'
     assert_response :success
@@ -144,10 +153,10 @@ class IssuesTest < ActionController::IntegrationTest
         :attributes => {:value => tester.id.to_s},
         :content => tester.name
       }
-    
+
     # Create issue
     assert_difference 'Issue.count' do
-      post '/projects/ecookbook/issues', 
+      post '/projects/ecookbook/issues',
         :issue => {
           :tracker_id => '1',
           :priority_id => '4',
@@ -157,7 +166,7 @@ class IssuesTest < ActionController::IntegrationTest
     end
     issue = Issue.first(:order => 'id DESC')
     assert_response 302
-    
+
     # Issue view
     follow_redirect!
     assert_tag :th,
@@ -174,7 +183,7 @@ class IssuesTest < ActionController::IntegrationTest
         :attributes => {:value => tester.id.to_s, :selected => 'selected'},
         :content => tester.name
       }
-    
+
     # Update issue
     new_tester = users[1]
     assert_difference 'Journal.count' do
@@ -185,7 +194,7 @@ class IssuesTest < ActionController::IntegrationTest
         }
     end
     assert_response 302
-    
+
     # Issue view
     follow_redirect!
     assert_tag :content => 'Tester',
@@ -197,4 +206,23 @@ class IssuesTest < ActionController::IntegrationTest
         }
       }
   end
+
+  def test_update_using_invalid_http_verbs
+    subject = 'Updated by an invalid http verb'
+
+    get '/issues/update/1', {:issue => {:subject => subject}}, credentials('jsmith')
+    assert_response 404
+    assert_not_equal subject, Issue.find(1).subject
+
+    post '/issues/1', {:issue => {:subject => subject}}, credentials('jsmith')
+    assert_response 405
+    assert_not_equal subject, Issue.find(1).subject
+  end
+
+  def test_get_watch_should_be_invalid
+    assert_no_difference 'Watcher.count' do
+      get '/watchers/watch?object_type=issue&object_id=1', {}, credentials('jsmith')
+      assert_response 405
+    end
+  end
 end
