@@ -18,7 +18,7 @@
 require File.expand_path('../../test_helper', __FILE__)
 
 class ProjectsTest < ActionController::IntegrationTest
-  fixtures :projects, :users, :members
+  fixtures :projects, :users, :members, :enabled_modules
 
   def test_archive_project
     subproject = Project.find(1).children.first
@@ -26,7 +26,7 @@ class ProjectsTest < ActionController::IntegrationTest
     get "admin/projects"
     assert_response :success
     assert_template "admin/projects"
-    post "projects/archive", :id => 1
+    post "projects/1/archive"
     assert_redirected_to "/admin/projects"
     assert !Project.find(1).active?
 
@@ -35,10 +35,17 @@ class ProjectsTest < ActionController::IntegrationTest
     get "projects/#{subproject.id}"
     assert_response 403
 
-    post "projects/unarchive", :id => 1
+    post "projects/1/unarchive"
     assert_redirected_to "/admin/projects"
     assert Project.find(1).active?
     get "projects/1"
     assert_response :success
   end
+
+  def test_modules_should_not_allow_get
+    assert_no_difference 'EnabledModule.count' do
+      get '/projects/1/modules', {:enabled_module_names => ['']}, credentials('jsmith')
+      assert_response :method_not_allowed
+    end
+  end
 end
