@@ -17,11 +17,11 @@
 
 require File.expand_path('../../../test_helper', __FILE__)
 
-class IssuesHelperTest < HelperTestCase
+class IssuesHelperTest < ActionView::TestCase
   include ApplicationHelper
   include IssuesHelper
+  include ERB::Util
 
-  include ActionController::Assertions::SelectorAssertions
   fixtures :projects, :trackers, :issue_statuses, :issues,
            :enumerations, :users, :issue_categories,
            :projects_trackers,
@@ -31,24 +31,10 @@ class IssuesHelperTest < HelperTestCase
            :enabled_modules,
            :workflows
 
-  # Used by assert_select
-  def html_document
-    HTML::Document.new(@response.body)
-  end
-
   def setup
     super
     set_language_if_valid('en')
     User.current = nil
-    @response = ActionController::TestResponse.new
-  end
-
-  def controller
-    @controller ||= IssuesController.new
-  end
-
-  def request
-    @request ||= ActionController::TestRequest.new
   end
 
   def test_issue_heading
@@ -77,70 +63,68 @@ class IssuesHelperTest < HelperTestCase
   context "IssuesHelper#show_detail" do
     context "with no_html" do
       should 'show a changing attribute' do
-        @detail = JournalDetail.generate!(:property => 'attr', :old_value => '40', :value => '100', :prop_key => 'done_ratio')
+        @detail = JournalDetail.new(:property => 'attr', :old_value => '40', :value => '100', :prop_key => 'done_ratio')
         assert_equal "% Done changed from 40 to 100", show_detail(@detail, true)
       end
 
       should 'show a new attribute' do
-        @detail = JournalDetail.generate!(:property => 'attr', :old_value => nil, :value => '100', :prop_key => 'done_ratio')
+        @detail = JournalDetail.new(:property => 'attr', :old_value => nil, :value => '100', :prop_key => 'done_ratio')
         assert_equal "% Done set to 100", show_detail(@detail, true)
       end
 
       should 'show a deleted attribute' do
-        @detail = JournalDetail.generate!(:property => 'attr', :old_value => '50', :value => nil, :prop_key => 'done_ratio')
+        @detail = JournalDetail.new(:property => 'attr', :old_value => '50', :value => nil, :prop_key => 'done_ratio')
         assert_equal "% Done deleted (50)", show_detail(@detail, true)
       end
     end
 
     context "with html" do
       should 'show a changing attribute with HTML highlights' do
-        @detail = JournalDetail.generate!(:property => 'attr', :old_value => '40', :value => '100', :prop_key => 'done_ratio')
-        @response.body = show_detail(@detail, false)
+        @detail = JournalDetail.new(:property => 'attr', :old_value => '40', :value => '100', :prop_key => 'done_ratio')
+        html = show_detail(@detail, false)
 
-        assert_select 'strong', :text => '% Done'
-        assert_select 'i', :text => '40'
-        assert_select 'i', :text => '100'
+        assert_include '<strong>% Done</strong>', html
+        assert_include '<i>40</i>', html
+        assert_include '<i>100</i>', html
       end
 
       should 'show a new attribute with HTML highlights' do
-        @detail = JournalDetail.generate!(:property => 'attr', :old_value => nil, :value => '100', :prop_key => 'done_ratio')
-        @response.body = show_detail(@detail, false)
+        @detail = JournalDetail.new(:property => 'attr', :old_value => nil, :value => '100', :prop_key => 'done_ratio')
+        html = show_detail(@detail, false)
 
-        assert_select 'strong', :text => '% Done'
-        assert_select 'i', :text => '100'
+        assert_include '<strong>% Done</strong>', html
+        assert_include '<i>100</i>', html
       end
 
       should 'show a deleted attribute with HTML highlights' do
-        @detail = JournalDetail.generate!(:property => 'attr', :old_value => '50', :value => nil, :prop_key => 'done_ratio')
-        @response.body = show_detail(@detail, false)
+        @detail = JournalDetail.new(:property => 'attr', :old_value => '50', :value => nil, :prop_key => 'done_ratio')
+        html = show_detail(@detail, false)
 
-        assert_select 'strong', :text => '% Done'
-        assert_select 'strike' do
-          assert_select 'i', :text => '50'
-        end
+        assert_include '<strong>% Done</strong>', html
+        assert_include '<strike><i>50</i></strike>', html
       end
     end
 
     context "with a start_date attribute" do
       should "format the current date" do
-        @detail = JournalDetail.generate!(:property => 'attr', :old_value => '2010-01-01', :value => '2010-01-31', :prop_key => 'start_date')
+        @detail = JournalDetail.new(:property => 'attr', :old_value => '2010-01-01', :value => '2010-01-31', :prop_key => 'start_date')
         assert_match "01/31/2010", show_detail(@detail, true)
       end
 
       should "format the old date" do
-        @detail = JournalDetail.generate!(:property => 'attr', :old_value => '2010-01-01', :value => '2010-01-31', :prop_key => 'start_date')
+        @detail = JournalDetail.new(:property => 'attr', :old_value => '2010-01-01', :value => '2010-01-31', :prop_key => 'start_date')
         assert_match "01/01/2010", show_detail(@detail, true)
       end
     end
 
     context "with a due_date attribute" do
       should "format the current date" do
-        @detail = JournalDetail.generate!(:property => 'attr', :old_value => '2010-01-01', :value => '2010-01-31', :prop_key => 'due_date')
+        @detail = JournalDetail.new(:property => 'attr', :old_value => '2010-01-01', :value => '2010-01-31', :prop_key => 'due_date')
         assert_match "01/31/2010", show_detail(@detail, true)
       end
 
       should "format the old date" do
-        @detail = JournalDetail.generate!(:property => 'attr', :old_value => '2010-01-01', :value => '2010-01-31', :prop_key => 'due_date')
+        @detail = JournalDetail.new(:property => 'attr', :old_value => '2010-01-01', :value => '2010-01-31', :prop_key => 'due_date')
         assert_match "01/01/2010", show_detail(@detail, true)
       end
     end
@@ -183,7 +167,5 @@ class IssuesHelperTest < HelperTestCase
 
     should "test custom fields"
     should "test attachments"
-
   end
-
 end
