@@ -1,43 +1,35 @@
 # Redmine - project management software
-# Copyright (C) 2006-2009  Jean-Philippe Lang
+# Copyright (C) 2006-2011  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
 # as published by the Free Software Foundation; either version 2
 # of the License, or (at your option) any later version.
-# 
+#
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
-# 
+#
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 require File.expand_path('../../../../../test_helper', __FILE__)
 
+class Redmine::MenuManager::MenuHelperTest < ActionView::TestCase
 
-
-class Redmine::MenuManager::MenuHelperTest < HelperTestCase
   include Redmine::MenuManager::MenuHelper
-  include ActionController::Assertions::SelectorAssertions
+  include ERB::Util
   fixtures :users, :members, :projects, :enabled_modules
 
-  # Used by assert_select
-  def html_document
-    HTML::Document.new(@response.body)
-  end
-  
   def setup
-    super
-    @response = ActionController::TestResponse.new
+    setup_with_controller
     # Stub the current menu item in the controller
-    def @controller.current_menu_item
+    def current_menu_item
       :index
     end
   end
-  
 
   context "MenuManager#current_menu_item" do
     should "be tested"
@@ -61,20 +53,20 @@ class Redmine::MenuManager::MenuHelperTest < HelperTestCase
 
   def test_render_single_menu_node
     node = Redmine::MenuManager::MenuItem.new(:testing, '/test', { })
-    @response.body = render_single_menu_node(node, 'This is a test', node.url, false)
+    @output_buffer = render_single_menu_node(node, 'This is a test', node.url, false)
 
     assert_select("a.testing", "This is a test")
   end
 
   def test_render_menu_node
     single_node = Redmine::MenuManager::MenuItem.new(:single_node, '/test', { })
-    @response.body = render_menu_node(single_node, nil)
+    @output_buffer = render_menu_node(single_node, nil)
 
     assert_select("li") do
       assert_select("a.single-node", "Single node")
     end
   end
-  
+
   def test_render_menu_node_with_nested_items
     parent_node = Redmine::MenuManager::MenuItem.new(:parent_node, '/test', { })
     parent_node << Redmine::MenuManager::MenuItem.new(:child_one_node, '/test', { })
@@ -83,7 +75,7 @@ class Redmine::MenuManager::MenuHelperTest < HelperTestCase
       Redmine::MenuManager::MenuItem.new(:child_three_node, '/test', { }) <<
       Redmine::MenuManager::MenuItem.new(:child_three_inner_node, '/test', { })
 
-    @response.body = render_menu_node(parent_node, nil)
+    @output_buffer = render_menu_node(parent_node, nil)
 
     assert_select("li") do
       assert_select("a.parent-node", "Parent node")
@@ -98,12 +90,12 @@ class Redmine::MenuManager::MenuHelperTest < HelperTestCase
         end
       end
     end
-    
+
   end
 
   def test_render_menu_node_with_children
     User.current = User.find(2)
-    
+
     parent_node = Redmine::MenuManager::MenuItem.new(:parent_node,
                                                      '/test',
                                                      {
@@ -117,7 +109,7 @@ class Redmine::MenuManager::MenuHelperTest < HelperTestCase
                                                          children
                                                        }
                                                      })
-    @response.body = render_menu_node(parent_node, Project.find(1))
+    @output_buffer = render_menu_node(parent_node, Project.find(1))
 
     assert_select("li") do
       assert_select("a.parent-node", "Parent node")
@@ -156,7 +148,7 @@ class Redmine::MenuManager::MenuHelperTest < HelperTestCase
                                                        }
                                                      })
 
-    @response.body = render_menu_node(parent_node, Project.find(1))
+    @output_buffer = render_menu_node(parent_node, Project.find(1))
 
     assert_select("li") do
       assert_select("a.parent-node", "Parent node")
@@ -185,10 +177,10 @@ class Redmine::MenuManager::MenuHelperTest < HelperTestCase
                                                      })
 
     assert_raises Redmine::MenuManager::MenuError, ":children must be an array of MenuItems" do
-      @response.body = render_menu_node(parent_node, Project.find(1))
+      @output_buffer = render_menu_node(parent_node, Project.find(1))
     end
   end
-    
+
   def test_render_menu_node_with_incorrect_children
     parent_node = Redmine::MenuManager::MenuItem.new(:parent_node,
                                                      '/test',
@@ -197,7 +189,7 @@ class Redmine::MenuManager::MenuHelperTest < HelperTestCase
                                                      })
 
     assert_raises Redmine::MenuManager::MenuError, ":children must be an array of MenuItems" do
-      @response.body = render_menu_node(parent_node, Project.find(1))
+      @output_buffer = render_menu_node(parent_node, Project.find(1))
     end
 
   end
@@ -214,7 +206,7 @@ class Redmine::MenuManager::MenuHelperTest < HelperTestCase
     menu_items_for(menu_name) do |item|
       items_yielded << item
     end
-    
+
     assert_equal 3, items_yielded.size
   end
 
@@ -239,11 +231,11 @@ class Redmine::MenuManager::MenuHelperTest < HelperTestCase
     end
 
     User.current = User.find(2)
-    
+
     items = menu_items_for(menu_name, Project.find(1))
     assert_equal 2, items.size
   end
-  
+
   def test_menu_items_for_should_skip_items_that_fail_the_conditions
     menu_name = :test_menu_items_for_should_skip_items_that_fail_the_conditions
     Redmine::MenuManager.map menu_name do |menu|
@@ -254,9 +246,8 @@ class Redmine::MenuManager::MenuHelperTest < HelperTestCase
     end
 
     User.current = User.find(2)
-    
+
     items = menu_items_for(menu_name, Project.find(1))
     assert_equal 1, items.size
   end
-
 end
