@@ -1,30 +1,30 @@
 # Redmine - project management software
-# Copyright (C) 2006-2008  Jean-Philippe Lang
+# Copyright (C) 2006-2011  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
 # as published by the Free Software Foundation; either version 2
 # of the License, or (at your option) any later version.
-# 
+#
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
-# 
+#
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 require File.expand_path('../../../../../test_helper', __FILE__)
 
-class Redmine::WikiFormatting::MacrosTest < HelperTestCase
+class Redmine::WikiFormatting::MacrosTest < ActionView::TestCase
   include ApplicationHelper
   include ActionView::Helpers::TextHelper
   include ActionView::Helpers::SanitizeHelper
   extend ActionView::Helpers::SanitizeHelper::ClassMethods
-  
+
   fixtures :projects, :roles, :enabled_modules, :users,
-                      :repositories, :changesets, 
+                      :repositories, :changesets,
                       :trackers, :issue_statuses, :issues,
                       :versions, :documents,
                       :wikis, :wiki_pages, :wiki_contents,
@@ -35,10 +35,21 @@ class Redmine::WikiFormatting::MacrosTest < HelperTestCase
     super
     @project = nil
   end
-  
+
   def teardown
   end
-  
+
+  def test_macro_registration
+    Redmine::WikiFormatting::Macros.register do
+      macro :foo do |obj, args|
+        "Foo macro output"
+      end
+    end
+
+    text = "{{foo}}"
+    assert_equal '<p>Foo macro output</p>', textilizable(text)
+  end
+
   def test_macro_hello_world
     text = "{{hello_world}}"
     assert textilizable(text).match(/Hello world!/)
@@ -46,13 +57,18 @@ class Redmine::WikiFormatting::MacrosTest < HelperTestCase
     text = "!{{hello_world}}"
     assert_equal '<p>{{hello_world}}</p>', textilizable(text)
   end
-  
+
+  def test_macro_macro_list
+    text = "{{macro_list}}"
+    assert_match %r{<code>hello_world</code>}, textilizable(text)
+  end
+
   def test_macro_include
     @project = Project.find(1)
     # include a page of the current project wiki
     text = "{{include(Another page)}}"
     assert textilizable(text).match(/This is a link to a ticket/)
-    
+
     @project = nil
     # include a page of a specific project wiki
     text = "{{include(ecookbook:Another page)}}"
@@ -64,23 +80,23 @@ class Redmine::WikiFormatting::MacrosTest < HelperTestCase
     text = "{{include(unknowidentifier:somepage)}}"
     assert textilizable(text).match(/Page not found/)
   end
-  
+
   def test_macro_child_pages
     expected =  "<p><ul class=\"pages-hierarchy\">\n" +
                  "<li><a href=\"/projects/ecookbook/wiki/Child_1\">Child 1</a></li>\n" +
                  "<li><a href=\"/projects/ecookbook/wiki/Child_2\">Child 2</a></li>\n" +
                  "</ul>\n</p>"
-    
+
     @project = Project.find(1)
     # child pages of the current wiki page
     assert_equal expected, textilizable("{{child_pages}}", :object => WikiPage.find(2).content)
     # child pages of another page
     assert_equal expected, textilizable("{{child_pages(Another_page)}}", :object => WikiPage.find(1).content)
-    
+
     @project = Project.find(2)
     assert_equal expected, textilizable("{{child_pages(ecookbook:Another_page)}}", :object => WikiPage.find(1).content)
   end
-  
+
   def test_macro_child_pages_with_option
     expected =  "<p><ul class=\"pages-hierarchy\">\n" +
                  "<li><a href=\"/projects/ecookbook/wiki/Another_page\">Another page</a>\n" +
@@ -88,14 +104,18 @@ class Redmine::WikiFormatting::MacrosTest < HelperTestCase
                  "<li><a href=\"/projects/ecookbook/wiki/Child_1\">Child 1</a></li>\n" +
                  "<li><a href=\"/projects/ecookbook/wiki/Child_2\">Child 2</a></li>\n" +
                  "</ul>\n</li>\n</ul>\n</p>"
-    
+
     @project = Project.find(1)
     # child pages of the current wiki page
     assert_equal expected, textilizable("{{child_pages(parent=1)}}", :object => WikiPage.find(2).content)
     # child pages of another page
     assert_equal expected, textilizable("{{child_pages(Another_page, parent=1)}}", :object => WikiPage.find(1).content)
-    
+
     @project = Project.find(2)
     assert_equal expected, textilizable("{{child_pages(ecookbook:Another_page, parent=1)}}", :object => WikiPage.find(1).content)
   end
+
+  def test_macro_child_pages_without_wiki_page_should_fail
+    assert_match /can be called from wiki pages only/, textilizable("{{child_pages}}")
+  end
 end
