@@ -71,4 +71,19 @@ class NewsTest < ActiveSupport::TestCase
     10.times { projects(:projects_001).news.create(valid_news) }
     assert_equal 5, News.latest(users(:users_004)).size
   end
+
+  def test_attachments_should_be_visible
+    assert News.find(1).attachments_visible?(User.anonymous)
+  end
+
+  def test_attachments_should_be_deletable_with_manage_news_permission
+    manager = User.find(2)
+    assert News.find(1).attachments_deletable?(manager)
+  end
+
+  def test_attachments_should_not_be_deletable_without_manage_news_permission
+    manager = User.find(2)
+    Role.find_by_name('Manager').remove_permission!(:manage_news)
+    assert !News.find(1).attachments_deletable?(manager)
+  end
 end
